import os
import unittest

from marvelous import SqliteCache, api, exceptions


class TestCreators(unittest.TestCase):
    def setUp(self):
        pub = os.getenv("PUBLIC_KEY", "pub")
        priv = os.getenv("PRIVATE_KEY", "priv")
        self.m = api(
            public_key=pub,
            private_key=priv,
            cache=SqliteCache("tests/testing_mock.sqlite"),
        )

    def test_known_creator(self):
        jason = self.m.creator(11463)
        self.assertTrue(jason.first_name == "Jason")
        self.assertTrue(jason.last_name == "Aaron")
        self.assertTrue(jason.id == 11463)
        self.assertTrue(
            jason.thumbnail
            == "http://i.annihil.us/u/prod/marvel/i/mg/7/10/5cd9c7870670e.jpg"
        )

        self.assertTrue(16450 in [s.id for s in jason.series])

        self.assertTrue(len(jason.series[:5]) == 5)
        self.assertTrue(len(jason.series) == len([x for x in jason.series if x.id > 3]))

    def test_bad_creator(self):
        with self.assertRaises(exceptions.ApiError):
            self.m.creator(-1)

    def test_pulls_verbose(self):
        creators = self.m.creators_list(
            {
                "orderBy": "modified",
            }
        )

        self.assertGreater(len(creators.creator), 0)


if __name__ == "__main__":
    unittest.main()



from marshmallow import INCLUDE, Schema, fields, post_load, pre_load


class Events:
    def __init__(self, id=None, name=None, **kwargs):
        self.id = id
        self.name = name
        self.unknown = kwargs


class EventsSchema(Schema):
    id = fields.Str()
    name = fields.Str()

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        data["id"] = data["resourceURI"].split("/")[-1]
        return data

    @post_load
    def make(self, data, **kwargs):
        return Events(**data)



import os
import unittest

import marvelous


class TestComics(unittest.TestCase):
    def setUp(self):
        pub = os.getenv("PUBLIC_KEY", "pub")
        priv = os.getenv("PRIVATE_KEY", "priv")
        self.m = marvelous.api(
            public_key=pub,
            private_key=priv,
            cache=marvelous.SqliteCache("tests/testing_mock.sqlite"),
        )

    def test_pulls_verbose(self):
        week = self.m.comics(
            {
                "format": "comic",
                "formatType": "comic",
                "noVariants": True,
                "dateDescriptor": "thisWeek",
            }
        )

        self.assertTrue(len(week.comics) > 0)

    def test_pulls_simple(self):
        week = self.m.comics({"dateDescriptor": "thisWeek"})
        self.assertTrue(len(week.comics) > 0)

    def test_pulls_simpler(self):
        week = self.m.comics()
        self.assertTrue(len(week.comics) > 0)

    def test_known_comic(self):
        af15 = self.m.comics({"digitalId": 4746}).comics[0]
        self.assertTrue(af15.title == "Amazing Fantasy (1962) #15")
        self.assertTrue(af15.issue_number == 15)
        self.assertTrue(af15.description is None)
        self.assertTrue(af15.format == "Comic")
        self.assertTrue(af15.page_count == 36)
        self.assertTrue(af15.id == 16926)
        self.assertTrue("Spider-Man" in [c.name for c in af15.characters])
        self.assertFalse("Foo" in [c.name for c in af15.characters])
        self.assertTrue("Steve Ditko" in [s.name for s in af15.creators])
        self.assertFalse("Abe Lincoln" in [s.name for s in af15.creators])

    def test_invalid_isbn(self):
        """Sometimes Marvel API sends number for isbn"""
        murpg = self.m.comics({"id": 1143}).comics[0]
        self.assertEqual(murpg.isbn, "785110283")

    def test_invalid_diamond_code(self):
        """Sometimes Marvel API sends number for diamond code"""
        hulk = self.m.comics({"id": 27399}).comics[0]
        self.assertEqual(hulk.diamond_code, "0")


if __name__ == "__main__":
    unittest.main()



class ApiError(Exception):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


class AuthenticationError(ApiError):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


class CacheError(ApiError):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)



from marshmallow import INCLUDE, Schema, fields, post_load, pre_load

from . import events, exceptions, series, urls


class Character:
    def __init__(self, **kwargs):
        if "response" not in kwargs:
            kwargs["response"] = None

        for k, v in kwargs.items():
            setattr(self, k, v)


class CharactersSchema(Schema):
    id = fields.Int()
    name = fields.Str()
    description = fields.Str()
    modified = fields.DateTime()
    resourceURI = fields.Str(attribute="resource_uri")
    urls = fields.Nested(urls.UrlsSchema)
    thumbnail = fields.Url()
    # comics
    # stories
    events = fields.Nested(events.EventsSchema, many=True)
    series = fields.Nested(series.SeriesSchema, many=True)

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        if data.get("code", 200) != 200:
            raise exceptions.ApiError(data.get("status"))

        if "status" in data:
            data["data"]["results"][0]["response"] = data
            data = data["data"]["results"][0]

        if "thumbnail" in data:
            data[
                "thumbnail"
            ] = f"{data['thumbnail']['path']}.{data['thumbnail']['extension']}"

        if "events" in data:
            data["events"] = data["events"]["items"]

        if "series" in data:
            data["series"] = data["series"]["items"]

        data["id"] = data["resourceURI"].split("/")[-1]
        return data

    @post_load
    def make(self, data, **kwargs):
        return Character(**data)



import json
import sqlite3


class SqliteCache:
    def __init__(self, db_name="marvelous_cache.db"):
        self.con = sqlite3.connect(db_name)
        self.cur = self.con.cursor()
        self.cur.execute("CREATE TABLE IF NOT EXISTS responses (key, json)")

    def get(self, key):
        self.cur.execute("SELECT json FROM responses WHERE key = ?", (key,))
        result = self.cur.fetchone()

        if result:
            return json.loads(result[0])

        return None

    def store(self, key, value):
        self.cur.execute(
            "INSERT INTO responses(key, json) VALUES(?, ?)", (key, json.dumps(value))
        )
        self.con.commit()



from marshmallow import INCLUDE, Schema, fields, post_load, pre_load


class Urls:
    def __init__(
        self,
        digital_purchase_date=None,
        foc_date=None,
        onsale_date=None,
        unlimited_date=None,
        wiki=None,
        detail=None,
        **kwargs
    ):
        self.digital_purchase_date = digital_purchase_date
        self.foc_date = foc_date
        self.onsale_date = onsale_date
        self.unlimited_date = unlimited_date
        self.wiki = wiki
        self.detail = detail
        self.unknown = kwargs


class UrlsSchema(Schema):
    digitalPurchaseDate = fields.Url(attribute="digital_purchase_date")
    focDate = fields.Url(attribute="foc_date")
    onsaleDate = fields.Url(attribute="onsale_date")
    unlimitedDate = fields.Url(attribute="unlimited_date")
    # Should these go into a separate class like CharacterUrls?
    # For now let's put them here, but it may be something to consider to split them.
    wiki = fields.Url()
    detail = fields.Url()

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        return {d["type"]: d["url"] for d in data}

    @post_load
    def make(self, data, **kwargs):
        return Urls(**data)



import datetime
import os
import unittest

import marvelous
from marvelous.comics_list import ComicsList
from marvelous.series import Series


class TestSeries(unittest.TestCase):
    def setUp(self):
        pub = os.getenv("PUBLIC_KEY", "pub")
        priv = os.getenv("PRIVATE_KEY", "priv")
        self.m = marvelous.api(
            public_key=pub,
            private_key=priv,
            cache=marvelous.SqliteCache("tests/testing_mock.sqlite"),
        )

    def test_known_series(self):
        usms = self.m.series(466)
        self.assertTrue(usms.title == "Ultimate Spider-Man (2000 - 2009)")
        self.assertTrue(usms.id == 466)
        self.assertEqual(
            "http://i.annihil.us/u/prod/marvel/i/mg/6/c0/5149db8019dc9.jpg",
            usms.thumbnail,
        )
        self.assertEqual(2009, usms.endYear)
        self.assertEqual(2000, usms.startYear)
        self.assertEqual(
            datetime.datetime(
                2013,
                3,
                20,
                11,
                54,
                44,
                tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=72000)),
            ),
            usms.modified,
        )
        self.assertEqual(
            "In 2000, Marvel embarked on a bold new experiment, "
            "re-imagining some of their greatest heroes in the 21st century, "
            "beginning with Spider-Man! Writer Brian Michael Bendis along with "
            "artists Mark Bagley and Stuart Immonen invite you to discover the "
            "world of Peter Parker in a whole new way with the series that changed "
            "everything!",
            usms.description,
        )
        self.assertEqual("A", usms.rating)

        comics = usms.comics()
        self.assertTrue(23931 in [c.id for c in comics])

        self.assertTrue(len(comics[:5]) == 5)
        self.assertTrue(len(comics) == len([x for x in comics if x.id > 3]))

    def test_bad_series(self):
        with self.assertRaises(marvelous.exceptions.ApiError):
            self.m.series(-1)
        with self.assertRaises(marvelous.exceptions.ApiError):
            self.m.series(params={"bad": "filter"})

    def test_bad_response_data(self):
        with self.assertRaises(marvelous.exceptions.ApiError):
            ComicsList({"data": {"results": [{"modified": "potato"}]}})

    def test_all_series(self):
        # check known values
        series_single = self.m.series(
            params={"title": "Ultimate Spider-Man"}
        )  # don't include '(year - year)', it doesn't work
        series_list = list(series_single)  # unspool the iterator
        self.assertIsNotNone(series_single)
        self.assertEqual(1, len(series_list))
        self.assertIsInstance(series_list[0], Series)
        # check the filter works returning multiple items
        seriesType = "ongoing"
        startYear = 2009
        series_iter = self.m.series(
            params={"seriesType": seriesType, "startYear": startYear, "limit": 10}
        )
        for series in series_iter:
            self.assertIsInstance(series, Series)
            self.assertEqual(startYear, series.startYear)


if __name__ == "__main__":
    unittest.main()



import itertools

from marshmallow import ValidationError

from . import exceptions, series


class SeriesList:
    def __init__(self, response):
        self.series = []
        self.response = response

        schema = series.SeriesSchema()
        for series_dict in response["data"]["results"]:
            try:
                result = schema.load(series_dict)
            except ValidationError as error:
                raise exceptions.ApiError(error)

            self.series.append(result)

    def __iter__(self):
        return iter(self.series)

    def __len__(self):
        return len(self.series)

    def __getitem__(self, index):
        try:
            return next(itertools.islice(self.series, index, index + 1))
        except TypeError:
            return list(
                itertools.islice(self.series, index.start, index.stop, index.step)
            )



from marshmallow import INCLUDE, Schema, fields, post_load, pre_load

from . import character, creator, dates, events, series, urls


class Comic:
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class ComicSchema(Schema):
    id = fields.Int()
    digitalId = fields.Int(attribute="digital_id")
    title = fields.Str()
    issueNumber = fields.Int(attribute="issue_number")
    variantDescription = fields.Str(attribute="variant_description")
    description = fields.Str(allow_none=True)
    modified = fields.DateTime()
    isbn = fields.Str()
    up = fields.Str()
    diamondCode = fields.Str(attribute="diamond_code")
    ean = fields.Str()
    issn = fields.Str()
    format = fields.Str()
    pageCount = fields.Int(attribute="page_count")
    # textObjects
    resourceURI = fields.Url(attribute="resource_uri")
    urls = fields.Nested(urls.UrlsSchema)
    series = fields.Nested(series.SeriesSchema)
    # variants
    # collections
    # collectedIssues
    dates = fields.Nested(dates.DatesSchema)
    # prices
    # thumbnail
    images = fields.List(fields.Url)
    creators = fields.Nested(creator.CreatorsSchema, many=True)
    characters = fields.Nested(character.CharactersSchema, many=True)
    # stories
    events = fields.Nested(events.EventsSchema, many=True)

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        new_data = data

        # Marvel comic 1768, and maybe others, returns a modified of
        # "-0001-11-30T00:00:00-0500". The best way to handle this is
        # probably just to ignore it, since I don't know how to fix it.
        if new_data.get("modified", " ")[0] == "-":
            del new_data["modified"]

        if "events" in new_data:
            new_data["events"] = new_data["events"]["items"]

        if "creators" in new_data:
            new_data["creators"] = new_data["creators"]["items"]

        if "characters" in new_data:
            new_data["characters"] = new_data["characters"]["items"]

        if "images" in new_data:
            new_data["images"] = [
                "{}.{}".format(img["path"], img["extension"])
                for img in new_data["images"]
            ]

        if "isbn" in new_data:
            new_data["isbn"] = str(new_data["isbn"])

        if "diamondCode" in new_data:
            new_data["diamondCode"] = str(new_data["diamondCode"])

        return new_data

    @post_load
    def make(self, data, **kwargs):
        return Comic(**data)



from setuptools import find_packages, setup

version = "1.4.1"

setup(
    author_email="robert@rkuykendall.com",
    author="Robert Kuykendall and Contributors",
    classifiers=[
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python",
        ("Topic :: Internet :: WWW/HTTP :: Dynamic Content " ":: CGI Tools/Libraries"),
        "Topic :: Utilities",
    ],
    description="Marvel API wrapper for python.",
    install_requires=["marshmallow", "requests"],
    keywords="marvel api comics python",
    long_description=open("README.rst", "r").read(),
    name="marvelous",
    packages=find_packages(),
    url="http://github.com/rkuykendall/marvelous",
    version=version,
    zip_safe=True,
)



import datetime
import hashlib
import urllib.parse
from collections import OrderedDict

import requests
from marshmallow import ValidationError

from . import (
    character,
    characters_list,
    comics_list,
    creator,
    creators_list,
    exceptions,
    series,
    series_list,
)


class Session:
    api_url = "http://gateway.marvel.com:80/v1/public/{}"

    def __init__(self, public_key, private_key, cache=None):

        self.public_key = public_key
        self.private_key = private_key
        self.cache = cache

    def call(self, endpoint, params=None):
        if params is None:
            params = {}

        # Generate part of cache key before hash, apikey and timestamp added
        cache_params = ""
        if len(params) > 0:
            orderedParams = OrderedDict(sorted(params.items(), key=lambda t: t[0]))
            cache_params = "?{}".format(urllib.parse.urlencode(orderedParams))

        now_string = datetime.datetime.now().strftime("%Y-%m-%d%H:%M:%S")
        auth_hash = hashlib.md5()
        auth_hash.update(now_string.encode("utf-8"))
        auth_hash.update(self.private_key.encode("utf-8"))
        auth_hash.update(self.public_key.encode("utf-8"))

        params["hash"] = auth_hash.hexdigest()
        params["apikey"] = self.public_key
        params["ts"] = now_string

        url = self.api_url.format("/".join(str(e) for e in endpoint))
        cache_key = "{url}{cache_params}".format(url=url, cache_params=cache_params)

        if self.cache:
            try:
                cached_response = self.cache.get(cache_key)

                if cached_response is not None:
                    return cached_response
            except AttributeError as e:
                raise exceptions.CacheError(
                    "Cache object passed in is missing attribute: {}".format(repr(e))
                )

        response = requests.get(url, params=params)

        data = response.json()

        if "message" in data:
            raise exceptions.ApiError(data["message"])

        if self.cache and response.status_code == 200:
            try:
                self.cache.store(cache_key, data)
            except AttributeError as e:
                raise exceptions.CacheError(
                    "Cache object passed in is missing attribute: {}".format(repr(e))
                )

        return data

    def comics(self, params=None):
        if params is None:
            params = {}

        return comics_list.ComicsList(self.call(["comics"], params=params))

    def series(self, _id=None, params=None):
        result = None
        if _id:
            try:
                result = series.SeriesSchema().load(self.call(["series", _id]))
                result.session = self
            except ValidationError as error:
                raise exceptions.ApiError(error)
        elif params:
            try:
                api_call = self.call(["series"], params)

                if api_call.get("code", 200) != 200:
                    raise exceptions.ApiError(api_call.get("status"))

                result = series.SeriesSchema().load(
                    api_call.get("data", {}).get("results"), many=True
                )
                for r in result:
                    r.session = self
            except ValidationError as error:
                raise exceptions.ApiError(error)

        return result

    def series_list(self, params=None):
        if params is None:
            params = {}

        return series_list.SeriesList(self.call(["series"], params=params))

    def creator(self, _id):
        try:
            result = creator.CreatorsSchema().load(self.call(["creators", _id]))
        except ValidationError as error:
            raise exceptions.ApiError(error)

        result.session = self
        return result

    def creators_list(self, params=None):
        if params is None:
            params = {}

        return creators_list.CreatorsList(self.call(["creators"], params=params))

    def character(self, _id):
        try:
            result = character.CharactersSchema().load(self.call(["characters", _id]))
        except ValidationError as error:
            raise exceptions.ApiError(error)

        result.session = self
        return result

    def characters_list(self, params=None):
        if params is None:
            params = {}

        return characters_list.CharactersList(self.call(["characters"], params=params))



from marshmallow import INCLUDE, Schema, fields, post_load, pre_load

from . import exceptions


class Series:
    def __init__(self, **kwargs):
        if "response" not in kwargs:
            kwargs["response"] = None

        for k, v in kwargs.items():
            if k == "comics":
                continue
            setattr(self, k, v)

    def comics(self, params=None):
        from . import comics_list

        if params is None:
            params = {}

        return comics_list.ComicsList(
            self.session.call(["series", self.id, "comics"], params=params)
        )


class SeriesSchema(Schema):
    response = fields.Raw()
    id = fields.Int()
    resourceURI = fields.Str(attribute="resource_uri")
    title = fields.Str()
    modified = fields.DateTime()
    description = fields.Str(allow_none=True)
    thumbnail = fields.URL(allow_none=True)
    startYear = fields.Int(allow_none=True)
    endYear = fields.Int(allow_none=True)
    rating = fields.Str(allow_none=True)

    class Meta:
        unknown = INCLUDE

    @pre_load
    def process_input(self, data, **kwargs):
        if data.get("code", 200) != 200:
            raise exceptions.ApiError(data.get("status"))

        if "status" in data:
            data["data"]["results"][0]["response"] = data
            data = data["data"]["results"][0]

        # derive ID
        data["id"] = data["resourceURI"].split("/")[-1]
        # derive thumbnail
        thumb_dict = data.get("thumbnail", {})
        if thumb_dict:
            data["thumbnail"] = f"{thumb_dict['path']}.{thumb_dict['extension']}"
        else:
            data["thumbnail"] = None
        return data

    @post_load
    def make(self, data, **kwargs):
        return Series(**data)



import os
import unittest

from marvelous import SqliteCache, api, exceptions


class TestCharacters(unittest.TestCase):
    def setUp(self):
        pub = os.getenv("PUBLIC_KEY", "pub")
        priv = os.getenv("PRIVATE_KEY", "priv")
        self.m = api(
            public_key=pub,
            private_key=priv,
            cache=SqliteCache("tests/testing_mock.sqlite"),
        )

    def test_known_character(self):
        cap = self.m.character(1009220)
        self.assertTrue(cap.name == "Captain America")
        self.assertTrue(
            cap.resource_uri == "http://gateway.marvel.com/v1/public/characters/1009220"
        )
        self.assertTrue(
            cap.thumbnail
            == "http://i.annihil.us/u/prod/marvel/i/mg/3/50/537ba56d31087.jpg"
        )
        self.assertGreater(len(cap.series), 0)
        self.assertGreater(len(cap.events), 0)

    def test_bad_character(self):
        with self.assertRaises(exceptions.ApiError):
            self.m.character(-1)

    def test_pulls_verbose(self):
        characters = self.m.characters_list(
            {
                "orderBy": "modified",
            }
        )

        self.assertGreater(len(characters.character), 0)
